using System;
using System.Collections.Generic;
using System.Text;
using DartScoreboard.GUI.Controls;

namespace DartScoreboard.Data
{
    public class PlayerController
    {
        private string _name = string.Empty;
        private int _id = 0;
        private int _dartsThrownTotal = 0;
        private int _dartsRemaining = 0;
        private int _score = 0;
        private DartTracker _dartTracker;
        private PlayerDarts _playerDartsControl = null;
        private PlayerName _playerNameControl = null;

        public DartTracker DartTracker
        {
            get
            {
                if (_dartTracker == null)
                    _dartTracker = new DartTracker();

                return _dartTracker;
            }
            set { _dartTracker = value; }
        }

        public int ID
        {
            get { return _id; }
            set { _id = value; }
        }

        public string Username
        {
            get { return _name; }
            set { _name = value; }
        }

        public int Score
        {
            get { return _score; }
            set 
            { 
                _score = value;
                _playerNameControl.playerScore.Content = value.ToString();
            }
        }

        public int DartsRemaining
        {
            get { return _dartsRemaining; }
            set
            {
                _dartsRemaining = value;
                UpdateDartsRemaining();
            }
        }

        public int DartsThrownTotal
        {
            get { return _dartsThrownTotal; }
            set { _dartsThrownTotal = value; }
        }

        private void UpdateDartsRemaining()
        {
            _playerNameControl.dart1.Visibility = System.Windows.Visibility.Hidden;
            _playerNameControl.dart2.Visibility = System.Windows.Visibility.Hidden;
            _playerNameControl.dart3.Visibility = System.Windows.Visibility.Hidden;

            switch (_dartsRemaining)
            {
                case 0:
                    return;
                    break;
                case 1:
                    goto OneDart;
                    break;
                case 2:
                    goto TwoDarts;
                    break;
                case 3:
                    goto ThreeDarts;
                    break;
            }

            ThreeDarts:
                _playerNameControl.dart3.Visibility = System.Windows.Visibility.Visible;
            TwoDarts:
                _playerNameControl.dart2.Visibility = System.Windows.Visibility.Visible;
            OneDart:
                _playerNameControl.dart1.Visibility = System.Windows.Visibility.Visible;
        }

        public List<int> MarkDart(Darts dart, int number)
        {
            _dartsThrownTotal++;
            DartsRemaining--;
            List<int> dartMarks = new List<int>();

            switch (dart)
            {
                case Darts.Dart20:
                    dartMarks = _playerDartsControl.D20.MarkDarts(number);
                    break;
                case Darts.Dart19:
                    dartMarks = _playerDartsControl.D19.MarkDarts(number);
                    break;
                case Darts.Dart18:
                    dartMarks = _playerDartsControl.D18.MarkDarts(number);
                    break;
                case Darts.Dart17:
                    dartMarks = _playerDartsControl.D17.MarkDarts(number);
                    break;
                case Darts.Dart16:
                    dartMarks = _playerDartsControl.D16.MarkDarts(number);
                    break;
                case Darts.Dart15:
                    dartMarks = _playerDartsControl.D15.MarkDarts(number);
                    break;
                case Darts.DartBull:
                    dartMarks = _playerDartsControl.DBull.MarkDarts(number);
                    break;
            }

            DartTracker[dart] += number;

            return dartMarks;
        }

        public void UnMarkDart(Darts dart, List<int> dartMarks, int number)
        {
            _dartsThrownTotal--;
            DartsRemaining++;

            switch (dart)
            {
                case Darts.Dart20:
                    _playerDartsControl.D20.UnMarkDarts(dartMarks);
                    break;
                case Darts.Dart19:
                    _playerDartsControl.D19.UnMarkDarts(dartMarks);
                    break;
                case Darts.Dart18:
                    _playerDartsControl.D18.UnMarkDarts(dartMarks);
                    break;
                case Darts.Dart17:
                    _playerDartsControl.D17.UnMarkDarts(dartMarks);
                    break;
                case Darts.Dart16:
                    _playerDartsControl.D16.UnMarkDarts(dartMarks);
                    break;
                case Darts.Dart15:
                    _playerDartsControl.D15.UnMarkDarts(dartMarks);
                    break;
                case Darts.DartBull:
                    _playerDartsControl.DBull.UnMarkDarts(dartMarks);
                    break;
            }

            DartTracker[dart] -= number;
        }

        public void StartTurn()
        {
            DartsRemaining = 3;
        }

        public void UndoTurn()
        {
            DartsRemaining = 0;
        }

        public PlayerController(PlayerName playerNameControl, PlayerDarts playerDartsControl, string name, int id)
        {
            _playerDartsControl = playerDartsControl;
            _playerNameControl = playerNameControl;

            _id = id;
            _name = name;

            _playerNameControl.Username = name;

            _playerDartsControl.Visibility = System.Windows.Visibility.Visible;
            _playerNameControl.Visibility = System.Windows.Visibility.Visible;

            _playerDartsControl.Initialize();

            Score = 0;
        }

        public int GetCountDartsScoring()
        {
            int count;

            count = _dartTracker[Darts.DartBull] + _dartTracker[Darts.Dart20] +
                _dartTracker[Darts.Dart19] + _dartTracker[Darts.Dart18] + _dartTracker[Darts.Dart17] +
                _dartTracker[Darts.Dart16] + _dartTracker[Darts.Dart15];

            return count;
        }
    }
}
