using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media.Animation;
using System.Windows.Media.Media3D;

namespace DartScoreboard.GUI.Forms
{
    /// <summary>
    /// Interaction logic for Main.xaml
    /// </summary>

    public partial class Main : Window
    {
        enum Mode
        {
            Start,
            Scoreboard
        }

        private Mode _mode; 

        public Main()
        {
            InitializeComponent();
            //this.Cursor = Cursors.None;
            ucStart.GameStarted += OnGameStarted;
            ucScoreboard.GameFinished += ucScoreboard_GameFinished;
            ucScoreboard.GameReset += ucScoreboard_GameReset;

            _mode = Mode.Start;
        }

        void ucScoreboard_GameReset()
        {
            _mode = Mode.Start;

            ucStart.InGame = true;

            ChangeView();
        }

        void ucScoreboard_GameFinished()
        {
            _mode = Mode.Start;

            ucStart.InGame = false;

            ChangeView();
        }


        public void StartGame(List<DartDataSet.PlayerRow> playerRows)
        {
            _mode = Mode.Scoreboard;
            ucScoreboard.StartGame(playerRows);
            ChangeView();
        }


        private void OnGameStarted(List<DartDataSet.PlayerRow> playerRows)
        {
            StartGame(playerRows);
        }


        private void zoomOutStoryboardCompleted(object sender, EventArgs args)
        {
            if (_mode == Mode.Scoreboard)
            {
                ucStart.Visibility = Visibility.Hidden;
                ucScoreboard.Visibility = Visibility.Visible;
            }
            else
            {
                ucStart.Visibility = Visibility.Visible;
                ucScoreboard.Visibility = Visibility.Hidden;
            }

            Storyboard s = (Storyboard)this.Resources["ZoomInStoryboard"];
            s.Begin(this);

        }


        private void zoomInStoryboardCompleted(object sender, EventArgs e)
        {
            scrollViewerBorder.Visibility = Visibility.Visible;
        }


        private void ChangeView()
        {

            Point3DCollection points = new Point3DCollection();

            double ratio = myScrollViewer.ActualWidth / myScrollViewer.ActualHeight;

            points.Add(new Point3D(5, -5 * ratio, 0));
            points.Add(new Point3D(5, 5 * ratio, 0));
            points.Add(new Point3D(-5, 5 * ratio, 0));

            points.Add(new Point3D(-5, 5 * ratio, 0));
            points.Add(new Point3D(-5, -5 * ratio, 0));
            points.Add(new Point3D(5, -5 * ratio, 0));

            points.Add(new Point3D(-5, 5 * ratio, 0));
            points.Add(new Point3D(-5, -5 * ratio, 0));
            points.Add(new Point3D(5, -5 * ratio, 0));

            points.Add(new Point3D(5, -5 * ratio, 0));
            points.Add(new Point3D(5, 5 * ratio, 0));
            points.Add(new Point3D(-5, 5 * ratio, 0));

            myGeometry.Positions = points;
            myViewport3D.Width = 100;
            myViewport3D.Height = 100 * ratio;

            scrollViewerBorder.Visibility = Visibility.Hidden;

            Storyboard s = (Storyboard)this.Resources["ZoomOutStoryboard"];
            s.Begin(this);

        }  

    }
}