using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using DartScoreboard.Data;
using System.Speech.Synthesis;


namespace DartScoreboard.GUI.Forms
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>

    public partial class Scoreboard : System.Windows.Controls.UserControl
    {
        public delegate void GameResetDelegate();
        public event GameResetDelegate GameReset;

        public delegate void GameFinishedDelegate();
        public event GameFinishedDelegate GameFinished;

        private bool _doubleBullEnabled = true;
        private bool _isAnimating = false;
        private BaseGame _game = null;
        private Darts _lastDart;
        private int _lastDartNumber;
        private bool _dartAnimation = false;


        public Scoreboard()
        {
            //this.Cursor = Cursors.None;
            InitializeComponent();
        }

        public void StartGame(List<DartDataSet.PlayerRow> playerRows)
        {
            player4Name.Visibility = Visibility.Hidden;
            player3Name.Visibility = Visibility.Hidden;
            player2Name.Visibility = Visibility.Hidden;
            player1Name.Visibility = Visibility.Hidden;

            player4Darts.Visibility = Visibility.Hidden;
            player3Darts.Visibility = Visibility.Hidden;
            player2Darts.Visibility = Visibility.Hidden;
            player1Darts.Visibility = Visibility.Hidden;

            _game = new CricketGame();

            ListenToEvents(true);

            SetRound(1);

            switch (playerRows.Count)
            {
                case 1:
                    goto OnePlayer;
                    break;
                case 2:
                    goto TwoPlayers;
                    break;
                case 3:
                    goto ThreePlayers;
                    break;
                case 4:
                    goto FourPlayers;
                    break;
            }

            FourPlayers:
                _game.Players.Insert(0, new PlayerController(player4Name, player4Darts, playerRows[3].Username, playerRows[3].PlayerID));
            ThreePlayers:
                _game.Players.Insert(0, new PlayerController(player3Name, player3Darts, playerRows[2].Username, playerRows[2].PlayerID));
            TwoPlayers:
                _game.Players.Insert(0, new PlayerController(player2Name, player2Darts, playerRows[1].Username, playerRows[1].PlayerID));
            OnePlayer:
                _game.Players.Insert(0, new PlayerController(player1Name, player1Darts, playerRows[0].Username, playerRows[0].PlayerID));

            _game.Players[0].StartTurn();
        }

        private void ListenToEvents(bool listen)
        {
            if (listen)
            {
                _game.RoundChanged += _game_RoundChanged;
                _game.GameWon += _game_GameWon;
            }
            else
            {
                _game.RoundChanged -= _game_RoundChanged;
                _game.GameWon -= _game_GameWon;
            }
        }

        private void OnDartThrown(Darts dart, int number)
        {
            if (!_isAnimating)
            {
                _lastDart = dart;
                _lastDartNumber = number;
                _dartAnimation = true;

                BeginAnimation();

                if (dart == Darts.DartBull)
                {
                    if (number > 1 && _doubleBullEnabled)
                    {
                        AnimateDoubleBull();
                    }
                    else
                    {
                        AnimateBullseye();
                    }
                }
                else if (dart == Darts.DartOutOfBounds)
                {
                    AnimateOffBoard();
                }
                else
                {
                    switch (number)
                    {
                        case 1:
                            AnimateSingle(dart);
                            break;
                        case 2:
                            AnimateDouble(dart);
                            break;
                        case 3:
                            AnimateTriple(dart);
                            break;
                    }
                }
            }
        }

        private void BeginAnimation()
        {
            _isAnimating = true;

            Storyboard dartThrownAnimation = (Storyboard)FindResource("dartThrowAnimation");

            dartThrownAnimation.Begin(this);
        }

        private void EndAnimation()
        {
            _isAnimating = false;

            if (_dartAnimation)
            {
                UpdateGame();
            }

            _dartAnimation = false;
        }

        private void OnUndoLastDart()
        {
            SetRound(_game.UndoLastDart());
        }

        private void UpdateGame()
        {
            _game.UpdateGame(_lastDart, _lastDartNumber);
        }

        private void AnimateDoubleBull()
        {
            Storyboard doubleBullAnimation = (Storyboard)FindResource("doubleBullAnimation");

            doubleBullAnimation.Begin(this);
        }

        private void AnimateOffBoard()
        {
            Storyboard missAnimation = (Storyboard)FindResource("missAnimation");

            missAnimation.Begin(this);
        }

        private void AnimateBullseye()
        {
            Storyboard bullseyeAnimation = (Storyboard)FindResource("bullseyeAnimation");
            bullseyeAnimation.Begin(this);
        }

        private void AnimateSingle(Darts dart)
        {
            Random rnd = new Random();
            Storyboard singleAnimation;
            string dartNumber = ((int)dart).ToString();

            int randomNumber = rnd.Next(0, 100);

            if (randomNumber > 50)
            {
                singleAnimation = (Storyboard)FindResource("singleAnimation3D");
                cs01.Text = dartNumber;
                cs02.Text = dartNumber;
                cs03.Text = dartNumber;
                cs04.Text = dartNumber;
                cs05.Text = dartNumber;
                cs06.Text = dartNumber;
            }
            else
            {
                singleAnimation = (Storyboard)FindResource("singleAnimation");
                singleText.Text = dartNumber;
            }

            singleAnimation.Begin(this);
        }

        private void AnimateDouble(Darts dart)
        {
            Random rnd = new Random();
            Storyboard doubleAnimation;
            string dartNumber = ((int)dart).ToString();

            int randomNumber = rnd.Next(0, 100);

            if (randomNumber > 50)
            {
                doubleAnimation = (Storyboard)FindResource("doubleAnimation");
                doubleNumber.Text = dartNumber;
            }
            else
            {
                doubleAnimation = (Storyboard)FindResource("doubleAnimation2");
                doubleText2b.Text = "DOUBLE " + dartNumber;
                doubleText2.Text = doubleText2b.Text;
            }

            doubleAnimation.Begin(this);
        }

        private void AnimateTriple(Darts dart)
        {
            Storyboard tripleAnimation = (Storyboard)FindResource("tripleAnimation");

            string dartNumber = ((int)dart).ToString();
            tripleTextNumber.Text = dartNumber;
            tripleTextNumber2.Text = dartNumber;
            tripleTextNumber3.Text = dartNumber;

            tripleAnimation.Begin(this);
        }

        private void OnGameWon(PlayerController player)
        {
            Storyboard gameOverAnimation = (Storyboard)FindResource("gameOverAnimation");
            string message = player.Username + " WINS!";

            tbGameOver.Text = message;

            gameOverAnimation.Begin(this);

            Speak(message);

            ListenToEvents(false);

            _game = null;
        }

        private void OnRoundChanged(int round)
        {
            BeginAnimation();

            Storyboard roundAnimation = (Storyboard)FindResource("roundChangedAnimation");
            string message = "ROUND " + round.ToString();

            roundChangedText.Text = message;
            
            roundAnimation.Begin(this);

            Speak(message);

            SetRound(round);
        }

        private void SetRound(int round)
        {
            tbRound.Text = "ROUND " + round.ToString();
        }

        private void Speak(string message)
        {
            SpeechSynthesizer synth = new SpeechSynthesizer();
            synth.SpeakAsync(message);
        }

        #region Event Handlers

        void gameOverAnimationComplete(object sender, EventArgs e)
        {
            if (GameFinished != null)
                GameFinished();
        }

        void MenuClicked(object sender, EventArgs e)
        {
            if (GameReset != null)
                GameReset();
        }

        void UndoClicked(object sender, EventArgs e)
        {
            OnUndoLastDart();
        }


        void DartMissedClicked(object sender, EventArgs e)
        {
            OnDartThrown(Darts.DartOutOfBounds, 1);
        }

        void _game_GameWon(PlayerController player)
        {
            OnGameWon(player);
        }

        void _game_RoundChanged(int round)
        {
            OnRoundChanged(round);
        }

        void OnAnimationComplete(object sender, EventArgs e)
        {
            EndAnimation();
        }

        void Triple20Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart20, 3);
        }

        void Double20Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart20, 2);
        }

        void Single20Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart20, 1);
        }

        void Triple19Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart19, 3);
        }

        void Double19Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart19, 2);
        }

        void Single19Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart19, 1);
        }

        void Triple18Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart18, 3);
        }

        void Double18Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart18, 2);
        }

        void Single18Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart18, 1);
        }

        void Triple17Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart17, 3);
        }

        void Double17Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart17, 2);
        }

        void Single17Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart17, 1);
        }

        void Triple16Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart16, 3);
        }

        void Double16Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart16, 2);
        }

        void Single16Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart16, 1);
        }

        void Triple15Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart15, 3);
        }

        void Double15Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart15, 2);
        }

        void Single15Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart15, 1);
        }

        void Triple14Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart14, 3);
        }

        void Double14Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart14, 2);
        }

        void Single14Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart14, 1);
        }

        void Triple13Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart13, 3);
        }

        void Double13Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart13, 2);
        }

        void Single13Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart13, 1);
        }

        void Triple12Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart12, 3);
        }

        void Double12Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart12, 2);
        }

        void Single12Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart12, 1);
        }

        void Triple11Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart11, 3);
        }

        void Double11Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart11, 2);
        }

        void Single11Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart11, 1);
        }

        void Triple10Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart10, 3);
        }

        void Double10Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart10, 2);
        }

        void Single10Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart10, 1);
        }

        void Triple09Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart09, 3);
        }

        void Double09Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart09, 2);
        }

        void Single09Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart09, 1);
        }

        void Triple08Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart08, 3);
        }

        void Double08Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart08, 2);
        }

        void Single08Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart08, 1);
        }

        void Triple07Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart07, 3);
        }

        void Double07Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart07, 2);
        }

        void Single07Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart07, 1);
        }

        void Triple06Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart06, 3);
        }

        void Double06Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart06, 2);
        }

        void Single06Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart06, 1);
        }

        void Triple05Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart05, 3);
        }

        void Double05Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart05, 2);
        }

        void Single05Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart05, 1);
        }

        void Triple04Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart04, 3);
        }

        void Double04Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart04, 2);
        }

        void Single04Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart04, 1);
        }

        void Triple03Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart03, 3);
        }

        void Double03Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart03, 2);
        }

        void Single03Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart03, 1);
        }

        void Triple02Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart02, 3);
        }

        void Double02Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart02, 2);
        }

        void Single02Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart02, 1);
        }

        void Triple01Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart01, 3);
        }

        void Double01Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart01, 2);
        }

        void Single01Click(object sender, EventArgs e)
        {
            OnDartThrown(Darts.Dart01, 1);
        }

        void SingleBullClick(object sender, EventArgs e)
        {
            OnDartThrown(Darts.DartBull, 1);
        }

        void DoubleBullClick(object sender, EventArgs e)
        {
            OnDartThrown(Darts.DartBull, 2);
        }

        #endregion
    }
}