using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using DartScoreboard.GUI.Controls;

namespace DartScoreboard.GUI.Forms
{
    /// <summary>
    /// Interaction logic for Start.xaml
    /// </summary>

    public partial class Start : System.Windows.Controls.UserControl
    {

        public delegate void GameStartedDelegate(List<DartDataSet.PlayerRow> players);
        public event GameStartedDelegate GameStarted;

        private List<string> _numberOfPlayersList = new List<string>();
        private int _playerIndex = 0;

        private bool _inGame = false;

        public bool InGame
        {
            get { return _inGame; }
            set { _inGame = value; }
        }
        
        public Start()
        {
            InitializeComponent();
            this.Cursor = Cursors.None;
            _numberOfPlayersList.Add("1 PLAYER");
            _numberOfPlayersList.Add("2 PLAYERS");
            _numberOfPlayersList.Add("3 PLAYERS");
            _numberOfPlayersList.Add("4 PLAYERS");

            player1.LoadPlayers();
        }


        private void OnPlayerCountChange(int direction)
        {
            

            if (_playerIndex + direction == _numberOfPlayersList.Count)
            {
                _playerIndex = 0;
            }
            else if (_playerIndex + direction < 0)
            {
                _playerIndex = _numberOfPlayersList.Count - 1;
            }
            else
            {
                _playerIndex += direction;
            }

            tbPlayerCountSelect.Text = _numberOfPlayersList[_playerIndex];

            int numberOfPlayers = _playerIndex + 1;


            if (playerPanel.Children.Count < numberOfPlayers)
            {
                while (playerPanel.Children.Count < numberOfPlayers)
                {
                    playerPanel.Children.Add(new PlayerChooser());
                }
            }
            else if (playerPanel.Children.Count > numberOfPlayers)
            {
                while (playerPanel.Children.Count > numberOfPlayers)
                {
                    playerPanel.Children.RemoveAt(playerPanel.Children.Count - 1);
                }
            }

        }

        private void StartGame()
        {
            List<DartDataSet.PlayerRow> playersRows = new List<DartDataSet.PlayerRow>();

            foreach (UIElement element in playerPanel.Children)
            {
                if (element is PlayerChooser)
                {
                    playersRows.Add(((PlayerChooser)element).PlayerRow);
                }
            }

            if (GameStarted != null)
                GameStarted(playersRows);
        }

        void OnStart(object sender, EventArgs e)
        {
            StartGame();
            
        }

        void OnNumPlayersNext(object sender, EventArgs e)
        {
            OnPlayerCountChange(1);
        }

        void OnNumPlayersPrevious(object sender, EventArgs e)
        {
            OnPlayerCountChange(-1);
        }
    }
}