Imports System
Imports System.Windows 'uielement
Imports System.Windows.Media 'visual operations
Imports System.Windows.Media.Animation 'animation effect stuff
Imports System.Windows.Controls 'LayoutOverrideDecorator
Imports System.Collections.Generic
Imports System.Windows.Input

Namespace DartScoreboardVB.Animations
    Friend Class Particle
        Public Location As Point
        Public Velocity As Vector
        Public LifeTime As DateTime
        Public DeathTime As DateTime
        Public StartColor As Color
        Public EndColor As Color
        Public Diameter As Double
    End Class

    Public Class FireworkEffect
        Inherits OverlayRenderDecorator
        Private _particles As List(Of Particle) = New List(Of Particle)()
        Protected _random As Random = New Random()
        Private _bounceOffContainer As Boolean = False
        Private _gravity As Vector = New Vector(0.0, 10.0)
        Private _gravitateToMouse As Boolean = False
        Private _gravitateScale As Double = 0.1

        Private _mouseDropsParticles As Boolean = False
        Private _secondsUntilDrop As Double = 0
        Private _lastMousePosition As Point = New Point(0, 0)

        Private _timeTracker As TimeTracker


#Region "Dependency Properties"
        Public Shared RadiusProperty As DependencyProperty = _
            DependencyProperty.Register( _
                "RadiusBase", _
                GetType(Double), _
                GetType(FireworkEffect), _
                New FrameworkPropertyMetadata(2.5) _
            )


        Public Shared RadiusVariationProperty As DependencyProperty = _
            DependencyProperty.Register( _
                "RadiusVariation", _
                GetType(Double), _
                GetType(FireworkEffect), _
                New FrameworkPropertyMetadata(2.0) _
            )

        Public Shared StartColorProperty As DependencyProperty = _
            DependencyProperty.Register( _
                "StartColor", _
                GetType(Color), _
                GetType(FireworkEffect), _
                New FrameworkPropertyMetadata(Color.FromArgb(245, 255, 255, 255)) _
            )

        Public Shared EndColorProperty As DependencyProperty = _
            DependencyProperty.Register( _
                "EndColor", _
                GetType(Color), _
                GetType(FireworkEffect), _
                New FrameworkPropertyMetadata(Color.FromArgb(100, 128, 128, 128)) _
            )

        Public Shared StartColorVariationProperty As DependencyProperty = _
            DependencyProperty.Register( _
                "StartColorVariation", _
                GetType(Color), _
                GetType(FireworkEffect), _
                New FrameworkPropertyMetadata(Color.FromArgb(10, 55, 50, 20)) _
            )

        Public Shared EndColorVariationProperty As DependencyProperty = _
            DependencyProperty.Register( _
                "EndColorVariation", _
                GetType(Color), _
                GetType(FireworkEffect), _
                New FrameworkPropertyMetadata(Color.FromArgb(50, 20, 100, 20)) _
            )

        Public Shared MouseDropDelayProperty As DependencyProperty = _
            DependencyProperty.Register( _
                "MouseDropDelay", _
                GetType(Double), _
                GetType(FireworkEffect), _
                New FrameworkPropertyMetadata(0.1) _
            )

        Public Shared MouseDropDelayVariationProperty As DependencyProperty = _
            DependencyProperty.Register( _
                "MouseDropDelayVariation", _
                GetType(Double), _
                GetType(FireworkEffect), _
                New FrameworkPropertyMetadata(0.05) _
            )

        Public Shared ClickBurstSizeProperty As DependencyProperty = _
            DependencyProperty.Register( _
                "ClickBurstSize", _
                GetType(Integer), _
                GetType(FireworkEffect), _
                New FrameworkPropertyMetadata(30) _
            )
#End Region

#Region "Properties"
        Public Property Radius() As Double
            Get
                Return CType(GetValue(RadiusProperty), Double)
            End Get
            Set(ByVal Value As Double)

                SetValue(RadiusProperty, Value)
            End Set
        End Property

        Public Property RadiusVariation() As Double
            Get
                Return CType(GetValue(RadiusVariationProperty), Double)
            End Get
            Set(ByVal Value As Double)

                SetValue(RadiusVariationProperty, Value)
            End Set
        End Property

        Public Property StartColor() As Color
            Get
                Return CType(GetValue(StartColorProperty), Color)
            End Get
            Set(ByVal Value As Color)

                SetValue(StartColorProperty, Value)
            End Set
        End Property

        Public Property EndColor() As Color
            Get
                Return CType(GetValue(EndColorProperty), Color)
            End Get
            Set(ByVal Value As Color)

                SetValue(EndColorProperty, Value)
            End Set
        End Property

        Public Property StartColorVariation() As Color
            Get
                Return CType(GetValue(StartColorVariationProperty), Color)
            End Get
            Set(ByVal Value As Color)

                SetValue(StartColorVariationProperty, Value)
            End Set
        End Property

        Public Property EndColorVariation() As Color
            Get
                Return CType(GetValue(EndColorVariationProperty), Color)
            End Get
            Set(ByVal Value As Color)

                SetValue(EndColorVariationProperty, Value)
            End Set
        End Property

        Public Property BounceOffContainer() As Boolean
            Get
                Return _bounceOffContainer
            End Get
            Set(ByVal Value As Boolean)
                _bounceOffContainer = Value
            End Set
        End Property

        Public Property GravitateToMouse() As Boolean
            Get
                Return _gravitateToMouse
            End Get
            Set(ByVal Value As Boolean)
                _gravitateToMouse = Value
            End Set
        End Property

        Public Property GravitateScale() As Double
            Get
                Return _gravitateScale
            End Get
            Set(ByVal Value As Double)
                _gravitateScale = Value
            End Set
        End Property

        Public Property MouseDropsParticles() As Boolean
            Get
                Return _mouseDropsParticles
            End Get
            Set(ByVal Value As Boolean)
                _mouseDropsParticles = Value
            End Set
        End Property

        Public Property MouseDropDelay() As Double
            Get
                Return CType(GetValue(MouseDropDelayProperty), Double)
            End Get
            Set(ByVal Value As Double)

                SetValue(MouseDropDelayProperty, Value)
            End Set
        End Property

        Public Property MouseDropDelayVariation() As Double
            Get
                Return CType(GetValue(MouseDropDelayVariationProperty), Double)
            End Get
            Set(ByVal Value As Double)

                SetValue(MouseDropDelayVariationProperty, Value)
            End Set
        End Property

        Public Property ClickBurstSize() As Integer
            Get
                Return CType(GetValue(ClickBurstSizeProperty), Integer)
            End Get
            Set(ByVal Value As Integer)

                SetValue(ClickBurstSizeProperty, Value)
            End Set
        End Property
#End Region


        Public Sub New()

        End Sub

        Protected Overrides Sub OnAttachChild(ByVal child As UIElement)
            AddHandler CompositionTarget.Rendering, AddressOf OnFrameCallback
            'JOY: CompositionTarget.Rendering += OnFrameCallback()

            'child.PreviewMouseLeftButtonUp += OnMouseLeftButtonUp
            'child.PreviewMouseMove += OnMouseMove

            _timeTracker = New TimeTracker()

        End Sub

        Protected Overrides Sub OnDetachChild(ByVal child As UIElement)
            RemoveHandler CompositionTarget.Rendering, AddressOf OnFrameCallback

            'child.PreviewMouseLeftButtonUp =  OnMouseLeftButtonUp 
            'child.PreviewMouseMove =  OnMouseMove 

            _timeTracker = Nothing
        End Sub

        Protected Overrides Sub OnPropertyChanged(ByVal e As DependencyPropertyChangedEventArgs)
            If e.Property.Equals(Canvas.LeftProperty) Then
                StartRandomBursts(CType(e.NewValue, Double))
            End If
        End Sub

        Protected Sub OnFrameCallback(ByVal sender As Object, ByVal e As EventArgs)
            UpdateParticles(_timeTracker.Update())
        End Sub

        Protected Overridable Sub UpdateParticles(ByVal deltatime As Double)
            'drop particles from mouse position
            If (_mouseDropsParticles And IsMouseOver) Then
                _secondsUntilDrop -= deltatime
                If (_secondsUntilDrop < 0.0) Then
                    AddRandomBurst(_lastMousePosition - New Vector(Radius / 2.0, Radius / 2.0), 1)
                    _secondsUntilDrop = MouseDropDelay + (_random.NextDouble() * 2.0 - 1.0) * MouseDropDelayVariation
                End If
            End If

            If (_particles.Count > 0) Then
                InvalidateVisual()
            End If

            'update all particles
            Dim i As Integer = 0
            While i < _particles.Count
                '_particles[i]
                Dim p As Particle = _particles(i)

                If (_gravitateToMouse) Then
                    p.Velocity += (_lastMousePosition - p.Location) * _gravitateScale
                Else
                    p.Velocity += _gravity
                End If
                p.Location += p.Velocity * deltatime

                If (_bounceOffContainer) Then
                    Dim myRadius As Double = p.Diameter / 2.0
                    If (p.Location.X - myRadius < 0.0) Then
                        p.Location.X = myRadius
                        p.Velocity.X *= -0.9
                    ElseIf (p.Location.X > ActualWidth - myRadius) Then
                        p.Location.X = ActualWidth - myRadius
                        p.Velocity.X *= -0.9
                    End If

                    If (p.Location.Y - myRadius < 0.0) Then
                        p.Location.Y = myRadius
                        p.Velocity.Y *= -0.9
                    ElseIf (p.Location.Y > ActualHeight - myRadius) Then
                        p.Location.Y = ActualHeight - myRadius
                        p.Velocity.Y *= -0.9
                    End If
                End If

                'only increment counter for live particles
                If (_timeTracker.ElapsedTime > p.DeathTime) Then
                    _particles.RemoveAt(i)
                Else
                    i = i + 1
                End If
            End While
        End Sub

        Protected Overrides Sub OnOverlayRender(ByVal drawingContext As DrawingContext)
            Dim i As Integer
            For i = 0 To _particles.Count - 1 Step i + 1
                Dim p As Particle = _particles(i)

                'figure out where in the particles life we are
                Dim particlelife As Double = (_timeTracker.ElapsedTime - p.LifeTime).TotalSeconds / (p.DeathTime - p.LifeTime).TotalSeconds
                Dim currentcolor As Color = Color.Multiply(p.StartColor, CType((1.0 - particlelife), Single)) + Color.Multiply(p.EndColor, CType(particlelife, Single))
                Dim brush As Brush = New RadialGradientBrush(currentcolor, Color.FromArgb(0, currentcolor.R, currentcolor.G, currentcolor.B))

                Dim rect As RectangleGeometry = New RectangleGeometry(New Rect(New Point(p.Location.X - p.Diameter / 2.0, p.Location.Y - p.Diameter / 2.0), New Size(p.Diameter, p.Diameter)))
                drawingContext.DrawGeometry(brush, Nothing, rect)
            Next
        End Sub

        Public Sub StartRandomBursts(ByVal x As Double)
            Dim sp As System.Media.SoundPlayer = New System.Media.SoundPlayer()

            'JOY: Imports System.IO.Stream sound = Resource1.Arrow_Swoosh2)
            Dim sound As System.IO.Stream = DartScoreboardVB.Resource1.Arrow_Swoosh2
            sp.Stream = sound
            sp.Play()

            AddRandomBurst(New Point(x, CType(GetValue(Canvas.TopProperty), Double)))
        End Sub

        Public Sub AddRandomBurst()
            Dim point As Point = New Point(_random.NextDouble() * ActualWidth, _random.NextDouble() * ActualHeight)
            AddRandomBurst(point, ClickBurstSize)
        End Sub

        Public Sub AddRandomBurst(ByVal location As Point)
            AddRandomBurst(location, ClickBurstSize)
        End Sub

        Public Sub AddRandomBurst(ByVal location As Point, ByVal count As Integer)
            Dim i As Integer
            For i = 0 To count - 1 Step i + 1
                Dim p As Particle = New Particle()

                Dim myRadius As Double = Radius + (_random.NextDouble() * 2.0 - 1.0) * RadiusVariation
                Dim lifetime As Double = _random.NextDouble() * 3.0 + 1.0

                p.Location = location
                p.Velocity = New Vector(_random.NextDouble() * 200.0 - 100.0, _random.NextDouble() * -200 + 100.0)
                p.LifeTime = _timeTracker.ElapsedTime
                p.DeathTime = _timeTracker.ElapsedTime + TimeSpan.FromSeconds(lifetime)
                p.Diameter = 2.0 * myRadius

                Dim myStartColor As Color = StartColor
                Dim myStartColorVariation As Color = StartColorVariation
                Dim myEndColor As Color = EndColor
                Dim myEndColorVariation As Color = EndColorVariation

                Dim myStartRandColor As Color = Color.FromScRgb(StartColorVariation.ScA * CType(_random.NextDouble() * 2.0 - 1.0, Double), StartColorVariation.ScR * CType(_random.NextDouble() * 2.0 - 1.0, Double), StartColorVariation.ScG * CType(_random.NextDouble() * 2.0 - 1.0, Double), StartColorVariation.ScB * CType(_random.NextDouble() * 2.0 - 1.0, Single))
                Dim myEndRandColor As Color = Color.FromScRgb(EndColorVariation.ScA * CType(_random.NextDouble() * 2.0 - 1.0, Double), EndColorVariation.ScR * CType(_random.NextDouble() * 2.0 - 1.0, Double), EndColorVariation.ScG * CType(_random.NextDouble() * 2.0 - 1.0, Double), EndColorVariation.ScB * CType(_random.NextDouble() * 2.0 - 1.0, Single))

                p.StartColor = myStartColor + myStartRandColor
                p.EndColor = myEndColor + myEndRandColor
                _particles.Add(p)
            Next
        End Sub
    End Class
End Namespace

