Imports System
Imports System.Collections.Generic
Imports System.Text
Imports DartScoreboardVB.GUI.Controls

Namespace DartScoreboardVB.Data
    Public Class BaseGame

        Public Delegate Sub RoundChangedDelegate(ByVal round As Integer)
        Public RoundChanged As RoundChangedDelegate

        '//public delegate void LastRoundDelegate(int round);
        '//public event LastRoundDelegate LastRound;

        '//public delegate void RoundLimitDelegate(PlayerController player);
        '//public event RoundLimitDelegate RoundLimit;

        Public Delegate Sub GameWonDelegate(ByVal player As PlayerController)
        Public GameWon As GameWonDelegate

        Private Const MAX_ROUNDS As Integer = 30

        Private _players As List(Of PlayerController)
        Private _currentPlayerIndex As Integer
        Private _round As Integer
        Private _dartsClosed As DartsClosed
        Private _highScore As Integer = 0
        Private _dartHistory As Stack(Of DartHistoryItem) = Nothing
        Private _startTime As DateTime

        Public Property Players() As List(Of PlayerController)
            Get
                Return _players
            End Get
            Set(ByVal Value As List(Of PlayerController))
                _players = Value
            End Set
        End Property

        Public Property Round() As Integer
            Get
                Return _round
            End Get
            Set(ByVal Value As Integer)
                _round = Value

                OnRoundChanged()
            End Set
        End Property

        Public Property CurrentPlayerIndex() As Integer
            Get
                Return _currentPlayerIndex
            End Get
            Set(ByVal Value As Integer)
                _currentPlayerIndex = Value
            End Set
        End Property

        Public Sub New()
            _players = New List(Of PlayerController)()
            _currentPlayerIndex = 0
            _round = 1
            _dartsClosed = New DartsClosed()
            _dartHistory = New Stack(Of DartHistoryItem)()
            _startTime = DateTime.Now
        End Sub

        Public Sub OnPlayerChange()
            If _currentPlayerIndex + 1 < _players.Count Then
                _currentPlayerIndex = _currentPlayerIndex + 1
            Else
                _currentPlayerIndex = 0
                Round = Round + 1
            End If

            _players(_currentPlayerIndex).StartTurn()
        End Sub

        Public Function UndoLastDart() As Integer
            If _dartHistory.Count > 0 Then
                Dim dartHistoryItem As DartHistoryItem = _dartHistory.Pop()

                _dartsClosed(dartHistoryItem.Dart) = dartHistoryItem.IsClosed
                _players(dartHistoryItem.PlayerIndex).Score -= dartHistoryItem.Score
                _players(dartHistoryItem.PlayerIndex).UnMarkDart(dartHistoryItem.Dart, dartHistoryItem.DartMarks, dartHistoryItem.NumberThrown)

                If _currentPlayerIndex > dartHistoryItem.PlayerIndex Or dartHistoryItem.PlayerIndex + 1 = _players.Count Then
                    _players(_currentPlayerIndex).UndoTurn()
                End If

                _currentPlayerIndex = dartHistoryItem.PlayerIndex
                _round = dartHistoryItem.Round



                'Update High Scores
                UpdateHighScore()

                'return the round in case it has changed
            End If

            Return _round
        End Function

        Private Sub UpdateHighScore()
            Dim highScore As Integer = 0

            Dim player As PlayerController
            For Each player In _players
                If player.Score > highScore Then
                    highScore = player.Score
                End If
            Next
        End Sub

        Private Function UpdateScores(ByVal lastDartThrown As Darts, ByVal numberThrown As Integer) As Integer
            Dim scoreToAdd As Integer = 0

            If Not _dartsClosed(lastDartThrown) Then
                'can still score
                Dim numberOpen As Integer = 3 - _players(_currentPlayerIndex).DartTracker(lastDartThrown)
                Dim numberScoring As Integer = 0

                If numberOpen > 0 Then
                    numberScoring = numberThrown - numberOpen
                Else
                    numberScoring = numberThrown
                End If

                If numberScoring > 0 And Not CheckClosedStatus(lastDartThrown) Then
                    scoreToAdd = CType(lastDartThrown * numberScoring, Integer)
                    _players(_currentPlayerIndex).Score += scoreToAdd
                End If
            End If

            If _players(_currentPlayerIndex).Score > _highScore Then
                _highScore = _players(_currentPlayerIndex).Score
            End If

            Return scoreToAdd
        End Function


        Private Function CheckClosedStatus(ByVal lastDartThrown As Darts) As Boolean
            Dim closed As Boolean = True

            Dim player As PlayerController
            For Each player In _players
                If Not player.Equals(_players(CurrentPlayerIndex)) Then
                    If player.DartTracker(lastDartThrown) < 3 Then
                        closed = False
                        Exit For
                    End If
                End If
            Next

            Return closed
        End Function

        Private Sub UpdateClosedStatus(ByVal lastDartThrown As Darts)
            Dim closed As Boolean = True

            Dim player As PlayerController
            For Each player In _players
                If player.DartTracker(lastDartThrown) < 3 Then
                    closed = False
                    Exit For
                End If
            Next

            If (closed) Then
                _dartsClosed(lastDartThrown) = True
            End If
        End Sub

        Private Sub OnGameFinished()

            SaveGame()

            If Not GameWon Is Nothing Then
                GameWon(_players(_currentPlayerIndex))
            End If
        End Sub

        Private Sub SaveGame()
            Dim dgTA As DartDataSetTableAdapters.DartGameTableAdapter = New DartDataSetTableAdapters.DartGameTableAdapter()
            Dim dgpTA As DartDataSetTableAdapters.DartGamePlayersTableAdapter = New DartDataSetTableAdapters.DartGamePlayersTableAdapter()
            Dim gameID As Integer = dgTA.Insert(_players(_currentPlayerIndex).ID, _startTime, DateTime.Now)

            Dim player As PlayerController
            For Each player In _players
                dgpTA.Insert(gameID, player.ID, player.DartsThrownTotal, player.GetCountDartsScoring(), player.Score)
            Next
        End Sub

        Private Sub OnRoundChanged()
            If Not RoundChanged Is Nothing Then
                RoundChanged(_round)
            End If
        End Sub

        Private Function IsGameOver() As Boolean
            If (_players(_currentPlayerIndex).DartTracker(Darts.Dart20) >= 3 And _
                _players(_currentPlayerIndex).DartTracker(Darts.Dart19) >= 3 And _
                _players(_currentPlayerIndex).DartTracker(Darts.Dart18) >= 3 And _
                _players(_currentPlayerIndex).DartTracker(Darts.Dart17) >= 3 And _
                _players(_currentPlayerIndex).DartTracker(Darts.Dart16) >= 3 And _
                _players(_currentPlayerIndex).DartTracker(Darts.Dart15) >= 3 And _
                _players(_currentPlayerIndex).DartTracker(Darts.DartBull) >= 3 And _
                _players(_currentPlayerIndex).Score >= _highScore) Then
                Return True
            End If

            Return False
        End Function

        Public Sub UpdateGame(ByVal lastDartThrown As Darts, ByVal numberThrown As Integer)
            Dim score As Integer = 0
            Dim wasClosed As Boolean = _dartsClosed(lastDartThrown)

            If lastDartThrown <> Darts.DartOutOfBounds Then
                score = UpdateScores(lastDartThrown, numberThrown)
                UpdateClosedStatus(lastDartThrown)
            End If

            Dim dartMarks As List(Of Integer) = _players(_currentPlayerIndex).MarkDart(lastDartThrown, numberThrown)

            _dartHistory.Push(New DartHistoryItem(score, lastDartThrown, numberThrown, _
            wasClosed, _currentPlayerIndex, _round, dartMarks))

            Dim myIsGameOver As Boolean = IsGameOver()

            If Not myIsGameOver Then
                If _players(_currentPlayerIndex).DartsRemaining = 0 Then
                    OnPlayerChange()
                End If
            Else
                OnGameFinished()
            End If
        End Sub
    End Class
End Namespace
