Imports System
Imports System.Collections.Generic
Imports System.Text
Imports DartScoreboardVB.GUI.Controls


Namespace DartScoreboardVB.Data
    Public Class PlayerController
        Private _name As String = String.Empty
        Private _id As Integer = 0
        Private _dartsThrownTotal As Integer = 0
        Private _dartsRemaining As Integer = 0
        Private _score As Integer = 0
        Private _dartTracker As DartTracker
        Private _playerDartsControl As GUI.Controls.PlayerDarts = Nothing
        Private _playerNameControl As GUI.Controls.PlayerName = Nothing

        Public Property DartTracker() As DartTracker
            Get
                If _dartTracker Is Nothing Then
                    _dartTracker = New DartTracker()
                End If

                Return _dartTracker
            End Get
            Set(ByVal Value As DartTracker)
                _dartTracker = Value
            End Set
        End Property

        Public Property ID() As Integer
            Get
                Return _id
            End Get
            Set(ByVal Value As Integer)
                _id = Value
            End Set
        End Property

        Public Property Username() As String
            Get
                Return _name
            End Get
            Set(ByVal Value As String)
                _name = Value
            End Set
        End Property

        Public Property Score() As Integer
            Get
                Return _score
            End Get
            Set(ByVal Value As Integer)
                _score = Value
                _playerNameControl.playerScore.Content = Value.ToString()
            End Set
        End Property

        Public Property DartsRemaining() As Integer
            Get
                Return _dartsRemaining
            End Get
            Set(ByVal Value As Integer)
                _dartsRemaining = Value
                UpdateDartsRemaining()
            End Set
        End Property

        Public Property DartsThrownTotal() As Integer
            Get
                Return _dartsThrownTotal
            End Get
            Set(ByVal Value As Integer)
                _dartsThrownTotal = Value
            End Set
        End Property

        Private Sub UpdateDartsRemaining()
            _playerNameControl.dart1.Visibility = System.Windows.Visibility.Hidden
            _playerNameControl.dart2.Visibility = System.Windows.Visibility.Hidden
            _playerNameControl.dart3.Visibility = System.Windows.Visibility.Hidden

            Select Case _dartsRemaining
                Case 0
                    Return
                Case 1
                    GoTo OneDart
                Case 2
                    GoTo TwoDarts
                Case 3
                    GoTo ThreeDarts
            End Select

ThreeDarts:
            _playerNameControl.dart3.Visibility = System.Windows.Visibility.Visible
TwoDarts:
            _playerNameControl.dart2.Visibility = System.Windows.Visibility.Visible
OneDart:
            _playerNameControl.dart1.Visibility = System.Windows.Visibility.Visible
        End Sub

        Public Function MarkDart(ByVal dart As Darts, ByVal number As Integer) As List(Of Integer)
            _dartsThrownTotal = _dartsThrownTotal + 1
            DartsRemaining = DartsRemaining - 1
            Dim dartMarks As List(Of Integer) = New List(Of Integer)()

            Select Case dart
                Case Darts.Dart20
                    dartMarks = _playerDartsControl.GetD20.MarkDarts(number)
                Case Darts.Dart19
                    dartMarks = _playerDartsControl.GetD19.MarkDarts(number)
                Case Darts.Dart18
                    dartMarks = _playerDartsControl.GetD18.MarkDarts(number)
                Case Darts.Dart17
                    dartMarks = _playerDartsControl.GetD17.MarkDarts(number)
                Case Darts.Dart16
                    dartMarks = _playerDartsControl.GetD16.MarkDarts(number)
                Case Darts.Dart15
                    dartMarks = _playerDartsControl.GetD15.MarkDarts(number)
                Case Darts.DartBull
                    dartMarks = _playerDartsControl.GetDBull.MarkDarts(number)
            End Select

            DartTracker(dart) += number

            Return dartMarks
        End Function

        Public Sub UnMarkDart(ByVal dart As Darts, ByVal dartMarks As List(Of Integer), ByVal number As Integer)
            _dartsThrownTotal = _dartsThrownTotal - 1
            DartsRemaining = DartsRemaining + 1

            Select Case dart
                Case Darts.Dart20
                    _playerDartsControl.GetD20.UnMarkDarts(dartMarks)
                Case Darts.Dart19
                    _playerDartsControl.GetD19.UnMarkDarts(dartMarks)
                Case Darts.Dart18
                    _playerDartsControl.GetD18.UnMarkDarts(dartMarks)
                Case Darts.Dart17
                    _playerDartsControl.GetD17.UnMarkDarts(dartMarks)
                Case Darts.Dart16
                    _playerDartsControl.GetD16.UnMarkDarts(dartMarks)
                Case Darts.Dart15
                    _playerDartsControl.GetD15.UnMarkDarts(dartMarks)
                Case Darts.DartBull
                    _playerDartsControl.GetDBull.UnMarkDarts(dartMarks)
            End Select

            DartTracker(dart) -= number
        End Sub

        Public Sub StartTurn()
            DartsRemaining = 3
        End Sub

        Public Sub UndoTurn()
            DartsRemaining = 0
        End Sub

        Public Sub New(ByVal playerNameControl As GUI.Controls.PlayerName, ByVal playerDartsControl As GUI.Controls.PlayerDarts, ByVal name As String, ByVal id As Integer)
            _playerDartsControl = playerDartsControl
            _playerNameControl = playerNameControl

            _id = id
            _name = name

            _playerNameControl.Username = name

            _playerDartsControl.Visibility = System.Windows.Visibility.Visible
            _playerNameControl.Visibility = System.Windows.Visibility.Visible

            _playerDartsControl.Initialize()

            Score = 0
        End Sub

        Public Function GetCountDartsScoring() As Integer
            Dim count As Integer

            count = _dartTracker(Darts.DartBull) + _dartTracker(Darts.Dart20) + _
                _dartTracker(Darts.Dart19) + _dartTracker(Darts.Dart18) + _dartTracker(Darts.Dart17) + _
                _dartTracker(Darts.Dart16) + _dartTracker(Darts.Dart15)

            Return count
        End Function
    End Class
End Namespace

