' Interaction logic for Main.xaml
Namespace DartScoreboardVB.GUI.Forms

    Partial Public Class Main
        Inherits System.Windows.Window

        Enum Mode
            Start
            Scoreboard
        End Enum

        Private _mode As Mode

        Public Sub New()
            InitializeComponent()
            Me.Cursor = Cursors.None
            ucStart.GameStarted = [Delegate].Combine(ucStart.GameStarted, New Start.GameStartedDelegate(AddressOf OnGameStarted))
            ucScoreboard.GameFinished = [Delegate].Combine(ucScoreboard.GameFinished, New Scoreboard.GameFinishedDelegate(AddressOf ucScoreboard_GameFinished))
            ucScoreboard.GameReset = [Delegate].Combine(ucScoreboard.GameReset, New Scoreboard.GameResetDelegate(AddressOf ucScoreboard_GameReset))

            _mode = Mode.Start
        End Sub

        Private Sub ucScoreboard_GameReset()
            _mode = Mode.Start
            ucStart.InGame = True
            ChangeView()
        End Sub

        Private Sub ucScoreboard_GameFinished()
            _mode = Mode.Start
            ucStart.InGame = False
            ChangeView()
        End Sub


        Public Sub StartGame(ByVal playerRows As List(Of DartDataSet.PlayerRow))
            _mode = Mode.Scoreboard
            ucScoreboard.StartGame(playerRows)
            ChangeView()
        End Sub


        Private Sub OnGameStarted(ByVal playerRows As List(Of DartDataSet.PlayerRow))
            StartGame(playerRows)
        End Sub


        Private Sub zoomOutStoryboardCompleted(ByVal sender As Object, ByVal args As EventArgs)
            If _mode = Mode.Scoreboard Then
                ucStart.Visibility = Windows.Visibility.Hidden
                ucScoreboard.Visibility = Windows.Visibility.Visible
            Else
                ucStart.Visibility = Windows.Visibility.Visible
                ucScoreboard.Visibility = Windows.Visibility.Hidden
            End If

            Dim s As Animation.Storyboard = CType(Me.Resources("ZoomInStoryboard"), Animation.Storyboard)
            s.Begin(Me)
        End Sub


        Private Sub zoomInStoryboardCompleted(ByVal sender As Object, ByVal e As EventArgs)
            scrollViewerBorder.Visibility = Windows.Visibility.Visible
        End Sub


        Private Sub ChangeView()

            Dim points As Media3D.Point3DCollection = New Media3D.Point3DCollection()

            Dim ratio As Double = myScrollViewer.ActualWidth / myScrollViewer.ActualHeight

            points.Add(New Media3D.Point3D(5, -5 * ratio, 0))
            points.Add(New Media3D.Point3D(5, 5 * ratio, 0))
            points.Add(New Media3D.Point3D(-5, 5 * ratio, 0))

            points.Add(New Media3D.Point3D(-5, 5 * ratio, 0))
            points.Add(New Media3D.Point3D(-5, -5 * ratio, 0))
            points.Add(New Media3D.Point3D(5, -5 * ratio, 0))

            points.Add(New Media3D.Point3D(-5, 5 * ratio, 0))
            points.Add(New Media3D.Point3D(-5, -5 * ratio, 0))
            points.Add(New Media3D.Point3D(5, -5 * ratio, 0))

            points.Add(New Media3D.Point3D(5, -5 * ratio, 0))
            points.Add(New Media3D.Point3D(5, 5 * ratio, 0))
            points.Add(New Media3D.Point3D(-5, 5 * ratio, 0))

            myGeometry.Positions = points
            myViewport3D.Width = 100
            myViewport3D.Height = 100 * ratio

            scrollViewerBorder.Visibility = Windows.Visibility.Hidden

            Dim s As Animation.Storyboard = CType(Me.Resources("ZoomOutStoryboard"), Animation.Storyboard)
            s.Begin(Me)

        End Sub

    End Class
End Namespace