Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Documents
Imports System.Windows.Input
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Media.Imaging
Imports System.Windows.Shapes
Imports DartScoreboardVB.Data
Imports System.Speech.Synthesis


Namespace DartScoreboardVB.GUI.Forms
    '/ <summary>
    '/ Interaction logic for Window1.xaml
    '/ </summary>

    Partial Public Class Scoreboard
        Inherits System.Windows.Controls.UserControl
        Public Delegate Sub GameResetDelegate()
        Public GameReset As GameResetDelegate

        Public Delegate Sub GameFinishedDelegate()
        Public GameFinished As GameFinishedDelegate

        Private _doubleBullEnabled As Boolean = True
        Private _isAnimating As Boolean = False
        Private _game As Data.CricketGame = Nothing
        Private _lastDart As Data.Darts
        Private _lastDartNumber As Integer
        Private _dartAnimation As Boolean = False

        Public Sub New()
            Me.Cursor = Cursors.None
            InitializeComponent()
        End Sub

        Public Sub StartGame(ByVal playerRows As List(Of DartDataSet.PlayerRow))
            player4Name.Visibility = Windows.Visibility.Hidden
            player3Name.Visibility = Windows.Visibility.Hidden
            player2Name.Visibility = Windows.Visibility.Hidden
            player1Name.Visibility = Windows.Visibility.Hidden

            player4Darts.Visibility = Windows.Visibility.Hidden
            player3Darts.Visibility = Windows.Visibility.Hidden
            player2Darts.Visibility = Windows.Visibility.Hidden
            player1Darts.Visibility = Windows.Visibility.Hidden

            _game = New Data.CricketGame()

            ListenToEvents(True)

            SetRound(1)

            Select Case playerRows.Count
                Case 1
                    GoTo OnePlayer
                Case 2
                    GoTo TwoPlayers
                Case 3
                    GoTo ThreePlayers
                Case 4
                    GoTo FourPlayers
            End Select

FourPlayers:
            _game.Players.Insert(0, New Data.PlayerController(player4Name, player4Darts, playerRows(3).Username, playerRows(3).PlayerID))
ThreePlayers:
            _game.Players.Insert(0, New Data.PlayerController(player3Name, player3Darts, playerRows(2).Username, playerRows(2).PlayerID))
TwoPlayers:
            _game.Players.Insert(0, New Data.PlayerController(player2Name, player2Darts, playerRows(1).Username, playerRows(1).PlayerID))
OnePlayer:
            _game.Players.Insert(0, New Data.PlayerController(player1Name, player1Darts, playerRows(0).Username, playerRows(0).PlayerID))

            _game.Players(0).StartTurn()
        End Sub

        Private Sub ListenToEvents(ByVal listen As Boolean)
            If listen Then
                _game.RoundChanged = [Delegate].Combine(_game.RoundChanged, New Data.BaseGame.RoundChangedDelegate(AddressOf _game_RoundChanged))
                _game.GameWon = [Delegate].Combine(_game.GameWon, New Data.BaseGame.GameWonDelegate(AddressOf _game_GameWon))
            Else
                _game.RoundChanged = [Delegate].Remove(_game.RoundChanged, New Data.BaseGame.RoundChangedDelegate(AddressOf _game_RoundChanged))
                _game.GameWon = [Delegate].Remove(_game.GameWon, New Data.BaseGame.GameWonDelegate(AddressOf _game_GameWon))
            End If
        End Sub

        Private Sub OnDartThrown(ByVal dart As Data.Darts, ByVal number As Integer)
            If Not _isAnimating Then
                _lastDart = dart
                _lastDartNumber = number
                _dartAnimation = True

                BeginAnimation()

                If dart = Data.Darts.DartBull Then
                    If number > 1 And _doubleBullEnabled Then
                        AnimateDoubleBull()
                    Else
                        AnimateBullseye()
                    End If
                ElseIf dart = Data.Darts.DartOutOfBounds Then
                    AnimateOffBoard()
                Else
                    Select Case number
                        Case 1
                            AnimateSingle(dart)
                        Case 2
                            AnimateDouble(dart)
                        Case 3
                            AnimateTriple(dart)
                    End Select
                End If
            End If
        End Sub

        Private Sub BeginAnimation()
            _isAnimating = True

            Dim dartThrownAnimation As Storyboard = CType(FindResource("dartThrowAnimation"), Storyboard)

            dartThrownAnimation.Begin(Me)
        End Sub

        Private Sub EndAnimation()
            _isAnimating = False

            If _dartAnimation Then
                UpdateGame()
            End If

            _dartAnimation = False
        End Sub

        Private Sub OnUndoLastDart()
            SetRound(_game.UndoLastDart())
        End Sub

        Private Sub UpdateGame()
            _game.UpdateGame(_lastDart, _lastDartNumber)
        End Sub

        Private Sub AnimateDoubleBull()
            Dim doubleBullAnimation As Storyboard = CType(FindResource("doubleBullAnimation"), Storyboard)
            doubleBullAnimation.Begin(Me)
        End Sub

        Private Sub AnimateOffBoard()
            Dim missAnimation As Storyboard = CType(FindResource("missAnimation"), Storyboard)

            missAnimation.Begin(Me)
        End Sub

        Private Sub AnimateBullseye()
            Dim bullseyeAnimation As Storyboard = CType(FindResource("bullseyeAnimation"), Storyboard)
            bullseyeAnimation.Begin(Me)
        End Sub

        Private Sub AnimateSingle(ByVal dart As Data.Darts)
            Dim rnd As Random = New Random()
            Dim singleAnimation As Storyboard
            Dim dartNumber As String = (CType(dart, Integer)).ToString()

            Dim randomNumber As Integer = rnd.Next(0, 100)

            If randomNumber > 50 Then
                singleAnimation = CType(FindResource("singleAnimation3D"), Storyboard)
                cs01.Text = dartNumber
                cs02.Text = dartNumber
                cs03.Text = dartNumber
                cs04.Text = dartNumber
                cs05.Text = dartNumber
                cs06.Text = dartNumber
            Else
                singleAnimation = CType(FindResource("singleAnimation"), Storyboard)
                singleText.Text = dartNumber
            End If

            singleAnimation.Begin(Me)
        End Sub

        Private Sub AnimateDouble(ByVal dart As Data.Darts)
            Dim rnd As Random = New Random()
            Dim doubleAnimation As Storyboard
            Dim dartNumber As String = (CType(dart, Integer)).ToString()

            Dim randomNumber As Integer = rnd.Next(0, 100)

            If randomNumber > 50 Then
                doubleAnimation = CType(FindResource("doubleAnimation"), Storyboard)
                doubleNumber.Text = dartNumber
            Else
                doubleAnimation = CType(FindResource("doubleAnimation2"), Storyboard)
                doubleText2b.Text = "DOUBLE " + dartNumber
                doubleText2.Text = doubleText2b.Text
            End If

            doubleAnimation.Begin(Me)
        End Sub

        Private Sub AnimateTriple(ByVal dart As Data.Darts)
            Dim tripleAnimation As Storyboard = CType(FindResource("tripleAnimation"), Storyboard)

            Dim dartNumber As String = (CType(dart, Integer)).ToString()
            tripleTextNumber.Text = dartNumber
            tripleTextNumber2.Text = dartNumber
            tripleTextNumber3.Text = dartNumber

            tripleAnimation.Begin(Me)
        End Sub

        Private Sub OnGameWon(ByVal player As Data.PlayerController)
            Dim gameOverAnimation As Storyboard = CType(FindResource("gameOverAnimation"), Storyboard)
            Dim message As String = player.Username + " WINS!"

            tbGameOver.Text = message

            gameOverAnimation.Begin(Me)

            Speak(message)

            ListenToEvents(False)

            _game = Nothing
        End Sub

        Private Sub OnRoundChanged(ByVal round As Integer)
            BeginAnimation()

            Dim roundAnimation As Storyboard = CType(FindResource("roundChangedAnimation"), Storyboard)
            Dim message As String = "ROUND " + round.ToString()

            roundChangedText.Text = message

            roundAnimation.Begin(Me)

            Speak(message)

            SetRound(round)
        End Sub

        Private Sub SetRound(ByVal round As Integer)
            tbRound.Text = "ROUND " + round.ToString()
        End Sub

        Private Sub Speak(ByVal message As String)
            Dim synth As SpeechSynthesizer = New SpeechSynthesizer()
            synth.SpeakAsync(message)
        End Sub

#Region "Event Handlers"

        Private Sub gameOverAnimationComplete(ByVal sender As Object, ByVal e As EventArgs)
            If Not GameFinished Is Nothing Then
                GameFinished()
            End If
        End Sub

        Private Sub MenuClicked(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            If Not GameReset Is Nothing Then
                GameReset()
            End If
        End Sub

        Private Sub UndoClicked(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnUndoLastDart()
        End Sub


        Private Sub DartMissedClicked(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.DartOutOfBounds, 1)
        End Sub

        Private Sub _game_GameWon(ByVal player As Data.PlayerController)
            OnGameWon(player)
        End Sub

        Private Sub _game_RoundChanged(ByVal round As Integer)
            OnRoundChanged(round)
        End Sub

        Private Sub OnAnimationComplete(ByVal sender As Object, ByVal e As EventArgs)
            EndAnimation()
        End Sub

        Private Sub Triple20Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart20, 3)
        End Sub

        Private Sub Double20Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart20, 2)
        End Sub

        Private Sub Single20Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart20, 1)
        End Sub

        Private Sub Triple19Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart19, 3)
        End Sub

        Private Sub Double19Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart19, 2)
        End Sub

        Private Sub Single19Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart19, 1)
        End Sub

        Private Sub Triple18Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart18, 3)
        End Sub

        Private Sub Double18Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart18, 2)
        End Sub

        Private Sub Single18Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart18, 1)
        End Sub

        Private Sub Triple17Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart17, 3)
        End Sub

        Private Sub Double17Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart17, 2)
        End Sub

        Private Sub Single17Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart17, 1)
        End Sub

        Private Sub Triple16Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart16, 3)
        End Sub

        Private Sub Double16Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart16, 2)
        End Sub

        Private Sub Single16Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart16, 1)
        End Sub

        Private Sub Triple15Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart15, 3)
        End Sub

        Private Sub Double15Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart15, 2)
        End Sub

        Private Sub Single15Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart15, 1)
        End Sub

        Private Sub Triple14Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart14, 3)
        End Sub

        Private Sub Double14Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart14, 2)
        End Sub

        Private Sub Single14Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart14, 1)
        End Sub

        Private Sub Triple13Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart13, 3)
        End Sub

        Private Sub Double13Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart13, 2)
        End Sub

        Private Sub Single13Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart13, 1)
        End Sub

        Private Sub Triple12Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart12, 3)
        End Sub

        Private Sub Double12Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart12, 2)
        End Sub

        Private Sub Single12Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart12, 1)
        End Sub

        Private Sub Triple11Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart11, 3)
        End Sub

        Private Sub Double11Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart11, 2)
        End Sub

        Private Sub Single11Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart11, 1)
        End Sub

        Private Sub Triple10Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart10, 3)
        End Sub

        Private Sub Double10Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart10, 2)
        End Sub

        Private Sub Single10Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart10, 1)
        End Sub

        Private Sub Triple09Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart09, 3)
        End Sub

        Private Sub Double09Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart09, 2)
        End Sub

        Private Sub Single09Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart09, 1)
        End Sub

        Private Sub Triple08Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart08, 3)
        End Sub

        Private Sub Double08Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart08, 2)
        End Sub

        Private Sub Single08Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart08, 1)
        End Sub

        Private Sub Triple07Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart07, 3)
        End Sub

        Private Sub Double07Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart07, 2)
        End Sub

        Private Sub Single07Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart07, 1)
        End Sub

        Private Sub Triple06Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart06, 3)
        End Sub

        Private Sub Double06Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart06, 2)
        End Sub

        Private Sub Single06Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart06, 1)
        End Sub

        Private Sub Triple05Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart05, 3)
        End Sub

        Private Sub Double05Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart05, 2)
        End Sub

        Private Sub Single05Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart05, 1)
        End Sub

        Private Sub Triple04Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart04, 3)
        End Sub

        Private Sub Double04Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart04, 2)
        End Sub

        Private Sub Single04Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart04, 1)
        End Sub

        Private Sub Triple03Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart03, 3)
        End Sub

        Private Sub Double03Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart03, 2)
        End Sub

        Private Sub Single03Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart03, 1)
        End Sub

        Private Sub Triple02Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart02, 3)
        End Sub

        Private Sub Double02Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart02, 2)
        End Sub

        Private Sub Single02Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart02, 1)
        End Sub

        Private Sub Triple01Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart01, 3)
        End Sub

        Private Sub Double01Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart01, 2)
        End Sub

        Private Sub Single01Click(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.Dart01, 1)
        End Sub

        Private Sub SingleBullClick(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.DartBull, 1)
        End Sub

        Private Sub DoubleBullClick(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
            OnDartThrown(Data.Darts.DartBull, 2)
        End Sub

#End Region
    End Class
End Namespace
